"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Camera = void 0;
const log_1 = require("../log");
const delay_1 = require("../delay");
const serialport_1 = require("serialport");
const parser_readline_1 = require("@serialport/parser-readline");
class CameraSerialPortMock extends serialport_1.SerialPortMock {
    constructor(options, openCallback = null) {
        super(options, openCallback);
        this.log = (0, log_1.createLog)('mock');
        console.dir(this);
    }
    write(buffer) {
        super.write(buffer);
        this.log.info(`Received data: "${buffer}"`);
        if (buffer === 'i') {
            this.emit('i');
        }
        return true;
    }
}
class Camera {
    constructor() {
        this.ready = false;
        this.connected = false;
        this.serial = null;
        this.baud = 57600;
        this.next = null;
        this.port = null;
        this.log = (0, log_1.createLog)('camera');
        this.parser = new parser_readline_1.ReadlineParser({ delimiter: '\r\n' });
        this.begin();
    }
    async begin() {
        let ports = [];
        let selected = false;
        try {
            ports = await this.enumerate();
        }
        catch (err) {
            this.log.error('Error calling enumerate()', err);
        }
        if (ports.length > 0) {
            for (let port of ports) {
                this.log.info(`Found USB serial device: ${port} ${selected ? '*' : ''}`);
                selected = false;
            }
            try {
                await this.connect(ports[0]);
            }
            catch (err) {
                this.log.error(`Error connecting to ${ports[0]}`, err);
            }
        }
        else {
            this.log.warn(`No USB serial devices found, connecting to MOCK...`);
            try {
                await this.connectMock();
            }
            catch (err) {
                this.log.error(`Error connecting to MOCK USB serial device "/dev/fake"`, err);
            }
        }
        await (0, delay_1.delay)(3000);
        await this.verify();
    }
    filter(port) {
        if ((port.manufacturer + '').toLowerCase().indexOf('arduino') !== -1 ||
            (port.path + '').toLowerCase().indexOf('usbserial') !== -1 ||
            (port.path + '').toLowerCase().indexOf('usbmodem') !== -1 ||
            (port.path + '').toLowerCase().indexOf('ttyusb') !== -1) {
            return true;
        }
        return false;
    }
    async enumerate() {
        let listed = [];
        try {
            listed = await serialport_1.SerialPort.list();
        }
        catch (err) {
            this.log.error('Error listing serial ports', err);
        }
        return listed.filter(this.filter).map((port) => port.path);
    }
    async connect(port) {
        this.port = port;
        try {
            this.serial = new serialport_1.SerialPort({
                path: this.port,
                baudRate: this.baud,
            });
        }
        catch (err) {
            this.log.error('Error creating SerialPort object', err);
            return;
        }
        if (this.serial !== null) {
            this.serial.pipe(this.parser);
            this.parser.on('data', this.onData.bind(this));
        }
        return new Promise(function (resolve, reject) {
            this.serial.on('open', () => {
                this.log.info(`Connected to USB serial device ${this.port} @ ${this.baud} baud`);
                this.connected = true;
                return resolve(true);
            });
        }.bind(this));
    }
    async connectMock() {
        this.port = '/dev/fake';
        try {
            serialport_1.SerialPortMock.binding.createPort(this.port);
            this.serial = new CameraSerialPortMock({
                path: this.port,
                baudRate: this.baud
            });
        }
        catch (err) {
            this.log.error('Error creating SerialPortMock object', err);
            return;
        }
        if (this.serial !== null) {
            this.serial.pipe(this.parser);
            this.parser.on('data', this.onData.bind(this));
        }
        return new Promise(function (resolve, reject) {
            this.serial.on('open', () => {
                this.log.info(`Connected to MOCK USB serial device ${this.port} @ ${this.baud} baud`);
                this.connected = true;
                return resolve(true);
            });
        }.bind(this));
    }
    onData(data) {
        this.log.info(`Received data: "${data}"`);
        if (this.next !== null) {
            this.next(data);
        }
    }
    async verify() {
        try {
            await this.confirm('i', 'i');
            this.log.info(`Confirmed mcopy device`);
        }
        catch (err) {
            this.log.error(`Error connecting to mcopy device`, err);
            return;
        }
        try {
            await this.confirm('m', 'k');
            this.log.info(`Confirmed mcopy camera`);
        }
        catch (err) {
            this.log.error(`Error identifying device`, err);
            return;
        }
        this.ready = true;
        this.log.info(`Camera connected and ready`);
    }
    async confirm(cmd, res) {
        return new Promise(function (resolve, reject) {
            this.next = function (data) {
                this.next = null;
                if (data === res) {
                    return resolve(true);
                }
                else {
                    return reject(new Error(`Response ${data} !== ${res}`));
                }
            }.bind(this);
            this.log.info(`Send data: "${cmd}"`);
            this.serial.write(cmd, (err, results) => {
                if (err) {
                    this.log.error('Error writing to device', err);
                    return reject(err);
                }
            });
        }.bind(this));
    }
}
exports.Camera = Camera;
module.exports = { Camera };
//# sourceMappingURL=index.js.map