import { createLog } from '../log';
import type { Logger } from 'winston';
import { delay } from '../delay';

import { SerialPort, SerialPortMock } from 'serialport';
import { ReadlineParser } from '@serialport/parser-readline';
import { MockBinding } from '@serialport/binding-mock';

interface PortInfo {
    path: string;
    manufacturer: string | undefined;
    serialNumber: string | undefined;
    pnpId: string | undefined;
    locationId: string | undefined;
    productId: string | undefined;
    vendorId: string | undefined;
}

class CameraSerialPortMock extends SerialPortMock {
	private log : Logger;
	constructor (options : any, openCallback : ErrorCallback = null) {
		super(options, openCallback);
		this.log = createLog('mock');
		console.dir(this);
	}

	write (buffer : any) : boolean {
		super.write(buffer)
		this.log.info(`Received data: "${buffer}"`);
		if (buffer === 'i') {
			this.emit('i');
		}
		return true;
	}
}

export class Camera {
	private log : Logger;
	private parser : ReadlineParser; 
	private ready : boolean = false;
	private connected : boolean = false;
	private serial : SerialPort | CameraSerialPortMock = null;
	private baud : number = 57600;
	private next : Function = null;
	private port : string = null;

	constructor () {
		this.log = createLog('camera');
		this.parser = new ReadlineParser({ delimiter: '\r\n' });
		this.begin();
	}

	private async begin () {
		let ports : string[] = [];
		let selected : boolean = false;
		try {
			ports = await this.enumerate();
		} catch (err) {
			this.log.error('Error calling enumerate()', err);
		}
		if (ports.length > 0) {
			for (let port of ports) {
				this.log.info(`Found USB serial device: ${port} ${selected ? '*' : ''}`);
				selected = false;
			}
			try {
				await this.connect(ports[0]);
			} catch (err) {
				this.log.error(`Error connecting to ${ports[0]}`, err);
			}
		} else {
			this.log.warn(`No USB serial devices found, connecting to MOCK...`)
			try {
				await this.connectMock();
			} catch (err) {
				this.log.error(`Error connecting to MOCK USB serial device "/dev/fake"`, err);
			}
		}
		await delay(3000);
		await this.verify();
	}

	private filter (port : PortInfo) {
		if ((port.manufacturer + '').toLowerCase().indexOf('arduino') !== -1 ||
			(port.path + '').toLowerCase().indexOf('usbserial') !== -1 ||
			(port.path + '').toLowerCase().indexOf('usbmodem') !== -1 || 
			(port.path + '').toLowerCase().indexOf('ttyusb') !== -1) {
			return true;
		}
		return false;
	}

	private async enumerate () : Promise<string[]> {
		let listed : PortInfo[] = [];
		try {
			listed = await SerialPort.list();
		} catch (err) {
			this.log.error('Error listing serial ports', err);
		}
		return listed.filter(this.filter).map((port : PortInfo) => port.path);
	}

	private async connect (port : string) {
		this.port = port;
		try {
			this.serial = new SerialPort({
				path : this.port,
				baudRate: this.baud,
			});
		} catch (err) {
			this.log.error('Error creating SerialPort object', err);
			return;
		}
		if (this.serial !== null) {
			this.serial.pipe(this.parser);
			this.parser.on('data', this.onData.bind(this));
		}
		
		return new Promise(function (resolve : Function, reject : Function) {
			this.serial.on('open', () => {
  				this.log.info(`Connected to USB serial device ${this.port} @ ${this.baud} baud`);
  				this.connected = true;
  				return resolve(true);
			});
		}.bind(this));
	}

	private async connectMock () {
		this.port = '/dev/fake';
		try {
			SerialPortMock.binding.createPort(this.port);
			this.serial = new CameraSerialPortMock({ 
				path : this.port, 
				baudRate: this.baud 
			});
		} catch (err) {
			this.log.error('Error creating SerialPortMock object', err);
			return;
		}
		if (this.serial !== null) {
			this.serial.pipe(this.parser);
			this.parser.on('data', this.onData.bind(this));
		}
		return new Promise(function (resolve : Function, reject : Function) {
			this.serial.on('open', () => {
  				this.log.info(`Connected to MOCK USB serial device ${this.port} @ ${this.baud} baud`);
  				this.connected = true;
  				return resolve(true);
			});
		}.bind(this));	
	}

	private onData (data : string) {
		this.log.info(`Received data: "${data}"`);
		if (this.next !== null) {
			this.next(data);
		}
	}

	private async verify () {
		try {
			await this.confirm('i', 'i');
			this.log.info(`Confirmed mcopy device`);
		} catch (err) {
			this.log.error(`Error connecting to mcopy device`, err);
			return;
		}
		try {
			await this.confirm('m', 'k');
			this.log.info(`Confirmed mcopy camera`);
		} catch (err) {
			this.log.error(`Error identifying device`, err);
			return;
		}
		this.ready = true;
		this.log.info(`Camera connected and ready`);
	}

	private async confirm (cmd : string, res : string) : Promise<boolean> {
		return new Promise (function (resolve : Function, reject: Function) {
			this.next = function (data : string) {
				this.next = null;
				if (data === res) {
					return resolve(true);
				} else {
					return reject(new Error(`Response ${data} !== ${res}`));
				}
			}.bind(this);
			this.log.info(`Send data: "${cmd}"`);
			this.serial.write(cmd, (err : any, results : any) => {
				if (err) {
					this.log.error('Error writing to device', err);
					return reject(err);
				}
			});
		}.bind(this));
	}
}

module.exports = { Camera };